#include "stdafx.h"
#include "afile.h"

////////////////////////////////////////////////////////////////
// Implementation of CAsciiFile
//

CAsciiFile::CAsciiFile()
{
}

CAsciiFile::~CAsciiFile()
{
}

// Convert 4-bit nibble to ascii char; e.g., 0x9 -> '9'
//
inline char TOHEX(BYTE b)
{ return b >= 10 ? ('A'+b-10) : ('0'+b); }

// Convert ascii char to 4-bit nibble; e.g., '9' -> 0x9
//
inline BYTE FROMHEX(char c)
{ return c >= 'A' ? (c-'A'+10) : (c-'0'); }

////////////////
// Read override: read chars from file and convert ASCII to binary. The
// implementation is typical C byte-twiddling (you just can't escape it).
//
UINT CAsciiFile::Read(void FAR* lpBuf, UINT nCount)
{
   LPBYTE pBytes = (LPBYTE)lpBuf;
   char buf[2];

   while (nCount-- > 0) {
      UINT nRead = CFile::Read(buf, 2);   // read 2 chars using CFile
      if (nRead<2)
         break;
      *pBytes++ = FROMHEX(buf[0])<<4 | FROMHEX(buf[1]);
   }
   return pBytes-(LPBYTE)lpBuf;
}

////////////////
// Write override: convert binary to ASCII and write to file.
// Inverse of Read.
//
void CAsciiFile::Write(const void FAR* lpBuf, UINT nCount)
{
   LPBYTE pBytes = (LPBYTE)lpBuf;
   char buf[2];

   while (nCount-- > 0) {
      BYTE b=*pBytes++;
      buf[0]=TOHEX(b>>4);     // conert 1st nibble to ASCII
      buf[1]=TOHEX(b & 0xF);  //..and second
      CFile::Write(buf, 2);   // write 2 bytes using CFile
   }
}

//////////////////
// The following functions should never be called by CArchive.
// The ASSERTs are there to make sure.
//
DWORD CAsciiFile::GetPosition() const
{
   ASSERT(FALSE);
   return 0;
}

CFile* CAsciiFile::Duplicate() const
{
   ASSERT(FALSE);
   return NULL;
}

void CAsciiFile::SetLength(DWORD dwNewLen)
{
   ASSERT(FALSE);
}

DWORD CAsciiFile::GetLength() const
{
   ASSERT(FALSE);
   return 0;
}

void CAsciiFile::LockRange(DWORD dwPos, DWORD dwCount)
{
   ASSERT(FALSE);
}

void CAsciiFile::UnlockRange(DWORD dwPos, DWORD dwCount)
{
   ASSERT(FALSE);
}

//////////////////
// Seek is special: see the column text
//
LONG CAsciiFile::Seek(LONG lOff, UINT nFrom)
{
   ASSERT(lOff==0 && nFrom==CFile::current);
   return 0;
}

